import pytest
from jgtutils import module

def test_function():
    result = module.function()  # replace 'function' with your actual function
    expected_result =  # add your expected result here
    assert result == expected_result